@@ -36,7 +36,7 @@ Misc:
 -----
     - connector, job: generic connector and job utilities for the ETL, see 'etl' for more information
     - odooconnector, odoojob: odoo connector and job utilities for the ETL, see 'etl' for more information
-    - odoosimplemigrate: a simple migrate class from odoo to odoo instance
+    - odoo_simple_migrate: a simple migrate class from odoo to odoo instance
     - csvconnector, csvjob: odoo connector and job utilities for the ETL, see 'etl' for more information
     - consumer: communicate with the remote python agent (ws_agent)
     - convert: convert data type and time (MB -> GB, seconds -> hours)
@@ -44,6 +44,7 @@ Misc:
     - data: a data wrapper and normalizer, the aim of this class to compute header (list) and lines (list pf lists)
     - date: work with string dates
     - default_value: work with default values on objects
+    - dataframe: a wrapper of pandas dataframe
     - df: a simple dataframe object with operations 'add' and 'remove', each data is a Serie object
     - env: new odoo environment inside odoo shell
     - serie: a simple Serie object
@@ -60,6 +61,7 @@ Misc:
     - random: generate some randoms
     - rpc: construct an odoo API RPC
     - sample: generate some sample data
+    - sequence: generate a sequence
     - sftp: tools for paramiko SFTP instance
     - slice: slice iterable objects
     - str: tools around the string objects, can also format a numeric value
@@ -413,11 +415,13 @@ def __odoo_simple_migrate():
         limit=1000,  #migrate the first 1000 records
         offset=60,   #migrate from the offset 60
         order='id desc',    #begin with the last created
-        fields=['name],     #fields to migrate
-        exclude_fields=[],  #exclude some fields
-        include_fields=[],  #force include some fields like create_date
-        many2one_with_names=[],  #as default is xmlid, it's possible to force using names on some many2one fields
-        debug=True,  #print to console the header and data sent to server
+        based_on_fields=['name],     #base on fields is used to chnage strategy from ImportExportXmlID to ReadCreateOrWrite
+        fields=['name],        #fields to migrate
+        exclude_fields=[],     #exclude some fields
+        include_fields=[],     #force include some fields like create_date
+        many2x_with_names=[],  #as default is xmlid, it's possible to force using names on some many2one fields
+        debug=True,            #print to console the header and data sent to server
+        require='exact'        #enum: 'exact', 'sup', 'exact_or_sup'
     )
     """
     Print.info(__odoo_simple_migrate.__doc__)
@@ -506,6 +510,8 @@ Some global format are :
     d.relativedelta(days=7, months=-2, years=1, hours=3, minutes=20, seconds=-30) #return a string after transformation
     d.apply(days=7, months=-2, years=1, hours=3, minutes=20, seconds=-30)         #return an object of Str
     d.set_format('%d') #change the default format
+    d.to_last_day()    #apply the last day of the month and return the object Date
+    d.to_fist_day()    #apply the first day of the month and return the object Date
     d.last_day()    #return a string of last day format: default format
     d.fist_day()    #return a string of first day format: default format
     d.to_datetime() #return a datetime object
@@ -551,6 +557,20 @@ def __df():
     """
     Print.info(__df.__doc__)
 
+@cli_help.command('dataframe')
+def __dataframe():
+    """DataFrame: a wrapper on pandas datagrames
+
+    from dyools import DataFrame
+    d = DataFrame(df=pandas_df, env=env_odoo, logger=log_odoo) #if no logger: take module logger
+    #d = DataFrame(df, env=False, logger=False)
+    d.to_odoo('res.partner', csv_path='res.partner.csv')
+    #d.to_odoo(model, by=100, csv_path=False, ctx={'tracking_disable': True})
+    d.enumerate_by('name', new_column='enum', concat_column='name_enum') #starts with 1 by group
+    #d.enumerate_by(column, new_column='enum', concat_column=False)
+    """
+    Print.info(__dataframe.__doc__)
+
 
 @cli_help.command('serie')
 def __serie():
@@ -579,13 +599,17 @@ def __env():
     from dyools import Env
     e = Env(env, odoo)
     e = Env(odoo=odoo, dbname='DEMO')
+    e = Env(gg=globals())
+
     # Instance operations
     r.dump_db(dest, zip=True)
     r.create_db(dbname=False, with_demo=False, language='fr_FR') #argument is optional
     r.drop_db(dbname=False) #argument is optional
     r.restore_db(path, drop=False)
     r.list_db()
+
     # Operations
+    e.recompute(e['sale.order'].search([]), ['amount_total', 'amount_untaxed', 'amount_tax'])
     e.get_addons(self, enterprise=False, core=False, extra=True, addons_path=False) #return tuple of lists (installed, uninstalled)
     addons_path is used to replace the default addons_path
     e.check_uninstalled_modules(self, enterprise=False, core=False, extra=True, addons_path=False) #exit -1 if all selected not installed
@@ -596,16 +620,16 @@ def __env():
     e.get(model, domain=[], limit=False, order=False, **kwargs) #get objects (recordset)
     e.update_xmlid(record, xmlid=False) #record should be valid, the function create the xmlid in the database (__import__)
     e.xmlid_to_object(XMLID)
+
     # Module management
     e.update_list()
     e.install('sale,account')
     e.upgrade('sale')
     e.uninstall('account')
+
     # Misc
     e.fields(self, model, fields=[])
     e.menus(self, debug=False, xmlid=False, action=False, user=False, crud=False)
-
-
     """
     Print.info(__env.__doc__)
 
@@ -898,6 +922,33 @@ First argument is the number of items, the second is the number of nested items
     Print.info(__sample.__doc__)
 
 
+
+@cli_help.command('sequence')
+def __sequence():
+    """Sequence: generate a sequence
+The get the next value, use the methods .next(), and to get the current value, use the property .value
+
+    from dyools import Sequence
+    s1 = Sequence()     #sequence starts with 0
+    s2 = Sequence(10)   #sequence starts with 10
+    s3 = Sequence(10, step=2)     #sequence starts with 10 with a step of 2
+    s4 = Sequence('E', padding=6) #sequence of characters with a padding of 6
+    # default params are :
+        start=1,
+        fill='0',
+        prefix='',
+        prefix_padding=0,
+        prefix_fill='0',
+        suffix='',
+        suffix_padding=0,
+        suffix_fill='0',
+        padding=0,
+        stop=None,
+        step=1
+    """
+    Print.info(__sequence.__doc__)
+
+
 @cli_help.command('sftp')
 def __sftp():
     """SFTP: tools for paramiko SFTP instance
@@ -976,6 +1027,8 @@ Accept a data like [['', 'name','age'], [1, 'John',30], [2, 'Luc',28]]
     tbl.remove_cols([2])
     tbl.remove_rows([2])
     tbl.get_flat()             #[['name', 1, 'John']]
+    new_tbl = Table.merge(tbl, tbl2, tbl3)   #merge many tables
+    new_tbl = Table.merge([tbl, tbl2, tbl3]) #merge many tables
     """
     Print.info(__table.__doc__)
 
